<?php

namespace Drupal\indieweb_microsub\Reader;

use Drupal\Core\StringTranslation\StringTranslationTrait;
use Drupal\indieweb_microsub\Entity\MicrosubChannelInterface;
use Drupal\reader\ReaderBase;

class Reader extends ReaderBase {

  use StringTranslationTrait;

  /**
   * {@inheritdoc}
   */
  public function getChannels() {
    if ($this->microsubIsInternal() && \Drupal::currentUser()->hasPermission('access reader')) {

      $ids = \Drupal::entityTypeManager()
        ->getStorage('indieweb_microsub_channel')
        ->getQuery()
        ->condition('status', 1)
        ->sort('weight', 'ASC')
        ->execute();

      $channels = [];

      $notifications = \Drupal::entityTypeManager()->getStorage('indieweb_microsub_item')->getUnreadCountByChannel(0);
      $channels[] = (object) [
        'uid' => 'notifications',
        'name' => 'Notifications',
        'unread' => (int) $notifications,
      ];

      /** @var \Drupal\indieweb_microsub\Entity\MicrosubChannelInterface $channel */
      foreach (\Drupal::entityTypeManager()->getStorage('indieweb_microsub_channel')->loadMultiple($ids) as $channel) {
        $unread = 0;

        $indicator = $indicator = $channel->getReadIndicator();
        if ($indicator == MicrosubChannelInterface::readIndicatorCount) {
          $unread = (int) $channel->getUnreadCount();
        }
        elseif ($indicator == MicrosubChannelInterface::readIndicatorNew) {
          $unread = (bool) $channel->getUnreadCount();
        }

        $channels[] = (object) [
          'uid' => $channel->id(),
          'name' => $channel->label(),
          'unread' => $unread,
        ];
      }

      return [
        'label' => t('IndieWeb'),
        'channels' => $channels,
      ];
    }
  }

  /**
   * {@inheritdoc}
   */
  public function getTimeline($id, $search = NULL) {
    if ($this->microsubIsInternal() && \Drupal::currentUser()->hasPermission('access reader')) {
      \Drupal::request()->request->set('channel', $id);
      /** @var \Drupal\indieweb_microsub\MicrosubClient\MicrosubClientInterface $client */
      $client = \Drupal::service('indieweb.microsub.client');
      $response = $client->getTimeline(TRUE, $search);
      if (!empty($response['response'])) {
        return $response['response'];
      }
    }

    return [];
  }

  /**
   * {@inheritdoc}
   */
  public function getTimelineActions($id) {
    return [[
      'action' => 'mark-read',
      'title' => $this->t('Mark read'),
    ]];
  }

  /**
   * {@inheritdoc}
   */
  public function doTimelineAction($action, $id) {
    if ($this->microsubIsInternal() && \Drupal::currentUser()->isAuthenticated()) {
      if ($action == 'mark-read') {
        \Drupal::entityTypeManager()->getStorage('indieweb_microsub_item')->changeReadStatus($id, 1, []);
      }
    }
  }

  /**
   * Returns whether the internal microsub channel is used.
   *
   * @return array|mixed|null
   */
  protected function microsubIsInternal() {
    return \Drupal::config('indieweb_microsub.settings')->get('microsub_internal');
  }

}
